import random
from character import base, table
from damage import chain

# -- retrieve the cost of the cheapest rank
def minimum_cost (attributes, race):
    minimum = 1000
    
    for attribute in attributes [:]:
        cost = table.fixed.cost (race, attribute)
        if minimum > cost: minimum = cost

    return minimum


# -- randomly level up a creature
def level_up (char, points):
    attributes = table.fixed.attributes (char.Race)
    number = len (attributes)
    
    # -- determine the minimum cost for leveling up attributes
    min_cost = minimum_cost (attributes, char.Race)
    
    while points >= min_cost:
        attribute = attributes[random.randrange (0, number)]
        
        # -- do not raise attributes that are too costly
        if points < table.fixed.cost (char.Race, attribute): 
            continue
        
        # -- do not exceed maximum
        if char.increase (attribute, 1):
            points = points - table.fixed.cost (char.Race, attribute)
        else:
            # -- attribute can't be raised any more
            attributes.remove (attribute)

            # -- are there attributes to raise left?
            number = len (attributes)
            if number == 0: return
            
            # -- recalculate minimum
            min_cost = minimum_cost (attributes, char.Race)

def print_creatures (a, b):
    print "                   A          B"
    print "  Level (Exp): %2i (%4i)   %2i (%4i)\n" % (a.Level, a.Experience, b.Level, b.Experience)
    print " Strength/Max: %2i / %2i     %2i / %2i" % (a.Strength, a.MaxStrength, b.Strength, b.MaxStrength)
    print " Hardness/Max: %2i / %2i     %2i / %2i" % (a.Hardness, a.MaxHardness, b.Hardness, b.MaxHardness)
    print "  Agility/Max: %2i / %2i     %2i / %2i" % (a.Agility, a.MaxAgility, b.Agility, b.MaxAgility)
    print "Willpower/Max: %2i / %2i     %2i / %2i" % (a.Willpower, a.MaxWillpower, b.Willpower, b.MaxWillpower)
    print "   Health/Max: %2i / %2i     %2i / %2i" % (a.Health, a.MaxHealth, b.Health, b.MaxHealth)
    print "    Power/Max: %2i / %2i     %2i / %2i" % (a.Power, a.MaxPower, b.Power, b.MaxPower)
    print "\n       Armour: %2i          %2i" % (a.PhysicalResistance, b.PhysicalResistance)
    print " Melee Attack: %2i          %2i" % (a.MeleeAttack, b.MeleeAttack)
    print "        Parry: %2i          %2i" % (a.Parry, b.Parry)
    print "-----------------------"

def create_creature (char, name):
    level = input ("Select level of creature %s (0 to quit): " % name)
    if level <= 0: return 0 
    
    exp = 100 * level * (level + 1) / 2
    char.increase_experience (exp)
    
    # -- add a random modifier
    attributes = table.fixed.attributes (char.Race)
    attribute = attributes[random.randrange (0, len (attributes))]
    
    char.add_modifier (attribute, 1)
    level_up (char, level * 5)
    
    # -- set derived attributes to maximum
    char.Health = char.MaxHealth
    char.Power = char.MaxPower

    return 1

# -- test code
a = base.base ("Wolf")
b = base.base ("Wolf")

while 1:
    # -- init
    if not create_creature (a, "A"): break
    if not create_creature (b, "B"): break
    
    print_creatures (a, b)
    
    a_health = a.Health
    b_health = b.Health
    
    # -- fight
    while a.Health > 0 and b.Health > 0:
        d_chain = chain.chain (a)
        d_chain.apply_effect (a, b)
        if b.Health > 0: 
            d_chain = chain.chain (b)
            d_chain.apply_effect (b, a)
    
        if a_health != a.Health or b_health != b.Health:
            print "Health: A (%i/%i)   B (%i/%i)" % (a.Health, a.MaxHealth, b.Health, b.MaxHealth) 
    
    # -- cleanup
    a.__init__(a.Race)
    b.__init__(b.Race)
